@@ -1,10 +1,12 @@
+from typing import Dict, List
+
 """Constants taken from the game and PathOfBuilding."""
 
 #: Skill tree node data is offset 7 bytes from the start.
-TREE_OFFSET = 7
+TREE_OFFSET: int = 7
 
 #: Skill tree IDs of keystones.
-KEYSTONE_IDS = {
+KEYSTONE_IDS: Dict[str, int] = {
     "acrobatics": 54307,
     "ancestral_bond": 41970,
     "arrow_dancing": 54922,
@@ -21,6 +23,7 @@ KEYSTONE_IDS = {
     "iron_reflexes": 10661,
     "mind_over_matter": 34098,
     "minion_instability": 18663,
+    "mortal_conviction": 9403,
     "necromantic_aegis": 45175,
     "pain_attunement": 31703,
     "perfect_agony": 23407,
@@ -30,32 +33,102 @@ KEYSTONE_IDS = {
     "runebinder": 42343,
     "unwavering_stance": 40907,
     "vaal_pact": 10808,
+    "wicked_ward": 23905,
     "zealots_oath": 63425,
 }
 
 # fmt: off
 #: Taken from DefaultMonsterStats.dat
-MONSTER_DAMAGE_TABLE = (5, 6, 6, 7, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 20, 21,
-                        23, 24, 26, 28, 30, 32, 34, 36, 39, 41, 44, 47, 50, 53, 56, 59,
-                        63, 67, 71, 75, 80, 84, 89, 94, 100, 106, 112, 118, 125, 131,
-                        139, 147, 155, 163, 172, 181, 191, 202, 212, 224, 236, 248, 262,
-                        275, 290, 305, 321, 338, 355, 374, 393, 413, 434, 456, 480, 504,
-                        530, 556, 584, 614, 645, 677, 711, 746, 783, 822, 862, 905, 949,
-                        996, 1045, 1096, 1149, 1205, 1264, 1325, 1390, 1457, 1527, 1601,
-                        1678, 1758)
+MONSTER_DAMAGE_TABLE: List[int] = [5, 6, 6, 7, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17,
+                                   18, 20, 21, 23, 24, 26, 28, 30, 32, 34, 36, 39, 41,
+                                   44, 47, 50, 53, 56, 59, 63, 67, 71, 75, 80, 84, 89,
+                                   94, 100, 106, 112, 118, 125, 131, 139, 147, 155, 163,
+                                   172, 181, 191, 202, 212, 224, 236, 248, 262, 275,
+                                   290, 305, 321, 338, 355, 374, 393, 413, 434, 456,
+                                   480, 504, 530, 556, 584, 614, 645, 677, 711, 746,
+                                   783, 822, 862, 905, 949, 996, 1045, 1096, 1149, 1205,
+                                   1264, 1325, 1390, 1457, 1527, 1601, 1678, 1758]
 #: Taken from DefaultMonsterStats.dat
-MONSTER_LIFE_TABLE = (15, 18, 21, 25, 29, 33, 38, 43, 49, 55, 61, 68, 76, 85, 94, 104,
-                      114, 126, 138, 152, 166, 182, 199, 217, 236, 257, 280, 304, 331,
-                      359, 389, 422, 456, 494, 534, 577, 624, 673, 726, 783, 844, 910,
-                      980, 1055, 1135, 1221, 1313, 1411, 1516, 1629, 1749, 1878, 2015,
-                      2162, 2319, 2486, 2665, 2857, 3061, 3279, 3512, 3760, 4025, 4308,
-                      4610, 4932, 5276, 5642, 6033, 6449, 6894, 7367, 7872, 8410, 8984,
-                      9595, 10246, 10940, 11679, 12466, 13304, 14198, 15149, 16161,
-                      17240, 18388, 19610, 20911, 22296, 23770, 25338, 27007, 28784,
-                      30673, 32684, 34823, 37098, 39519, 42093, 44831)
-# fmt: on
+MONSTER_LIFE_TABLE: List[int] = [15, 18, 21, 25, 29, 33, 38, 43, 49, 55, 61, 68, 76, 85,
+                                 94, 104, 114, 126, 138, 152, 166, 182, 199, 217, 236,
+                                 257, 280, 304, 331, 359, 389, 422, 456, 494, 534, 577,
+                                 624, 673, 726, 783, 844, 910, 980, 1055, 1135, 1221,
+                                 1313, 1411, 1516, 1629, 1749, 1878, 2015, 2162, 2319,
+                                 2486, 2665, 2857, 3061, 3279, 3512, 3760, 4025, 4308,
+                                 4610, 4932, 5276, 5642, 6033, 6449, 6894, 7367, 7872,
+                                 8410, 8984, 9595, 10246, 10940, 11679, 12466, 13304,
+                                 14198, 15149, 16161, 17240, 18388, 19610, 20911, 22296,
+                                 23770, 25338, 27007, 28784, 30673, 32684, 34823, 37098,
+                                 39519, 42093, 44831]
+#: Vaal Skills with irregular names
+VAAL_SKILL_MAP: Dict[str, str] = {
+    "Vaal Breach": "Portal",
+    "Vaal Impurity of Fire": "Purity of Fire",
+    "Vaal Impurity of Ice": "Purity of Ice",
+    "Vaal Impurity of Lightning": "Purity of Lightning",
+}
+#: Mapping between internal skill IDs and their display names
+SKILL_MAP: Dict[str, str] = {
+    "Melee": "Default Attack",
+    "MeleeUseContactPoint": "Default Attack",
+    "GemDetonateMines": "Detonate Mines",
+    "Portal": "Portal",
+    "VaalBreach": "Vaal Breach",
+    "SupportUniqueMjolnerLightningSpellsCastOnHit": "Mjolner",
+    "SupportAuraDuration": "Blessing",
+    "SupportUniqueCosprisMaliceColdSpellsCastOnMeleeCriticalStrike": "Cospri's Malice",
+    "UniqueSupportGreaterVolley": "Greater Volley",
+    "RepeatingShockwave": "Abberath's Fury",
+    "BirdAspect": "Aspect of the Avian",
+    "CatAspect": "Aspect of the Cat",
+    "CrabAspect": "Aspect of the Crab",
+    "SpiderAspect": "Aspect of the Spider",
+    "BloodOffering": "Blood Offering",
+    "TriggeredBoneNova": "Bone Nova",
+    "ColdAegis": "Cold Aegis",
+    "TriggeredConsecrate": "Consecrate",
+    "TriggeredSummonLesserShrine": "Create Lesser Shrine",
+    "ChaosDegenAuraUnique": "Death Aura",
+    "DeathWalk": "Death Walk",
+    "TouchOfGod": "Doryani's Touch",
+    "ElementalAegis": "Elemental Aegis",
+    "SupportElementalPenetration": "Elemental Penetration",
+    "OnHitWhileCursedTriggeredCurseNova": "Elemental Warding",
+    "Envy": "Envy",
+    "FireAegis": "Fire Aegis",
+    "FireBurstOnHit": "Fire Burst",
+    "VaalAuraElementalDamageHealing": "Gluttony of Elements",
+    "IcestormUniqueStaff12": "Icestorm",
+    "TriggeredIcicleNova": "Icicle Burst",
+    "MerveilWarp": "Illusory Warp",
+    "LightningAegis": "Lightning Aegis",
+    "LightningSpell": "Lightning Bolt",
+    "UniqueAnimateWeapon": "Manifest Dancing Dervish",
+    "TriggeredMoltenStrike": "Molten Burst",
+    "TriggeredSummonSpider": "Raise Spiders",
+    "ShadeForm": "Shade Form",
+    "TriggeredShockedGround": "Shock Ground",
+    "SpiritBurst": "Spirit Burst",
+    "StormCascadeTriggered": "Storm Cascade",
+    "SummonBeastialRhoa": "Summon Bestial Rhoa",
+    "SummonBeastialSnake": "Summon Bestial Snake",
+    "SummonBeastialUrsa": "Summon Bestial Ursa",
+    "SummonEssenceSpirits": "Spectral Spirits",
+    "SummonHarbingerOfTheArcane": "Summon Harbinger of the Arcane",
+    "SummonHarbingerOfBrutality": "Summon Harbinger of Brutality",
+    "SummonHarbingerOfDirections": "Summon Harbinger of Directions",
+    "SummonHarbingerOfFocus": "Summon Harbinger of Focus",
+    "SummonHarbingerOfStorms": "Summon Harbinger of Storms",
+    "SummonHarbingerOfTime": "Summon Harbinger of Time",
+    "TriggeredSummonGhostOnKill": "Summon Phantasm",
+    "SummonRigwaldsPack": "Summon Spectral Wolf",
+    "SummonVoidSphere": "Summon Volatile Anomaly",
+    "TentacleSmash": "Tentacle Whip",
+    "AvianTornado": "Tornado",
+    "VoidGaze": "Void Gaze",
+}
 #: Mapping between PathOfBuilding's export format and this API.
-CONFIG_MAP = {
+CONFIG_MAP: Dict[str, str] = {
     "resistancePenalty": "resistance_penalty",
     "enemyLevel": "enemy_level",
     "enemyPhysicalHit": "enemy_physical_hit_damage",
@@ -76,21 +149,31 @@ CONFIG_MAP = {
     "aspectOfTheSpiderWebStacks": "aspect_of_the_spider_web_stacks",
     "bannerPlanted": "banner_planted",
     "bannerStages": "banner_stages",
+    "bladestormInBloodstorm": "in_bloodstorm",
+    "bladestormInSandstorm": "in_sandstorm",
     "brandAttachedToEnemy": "brand_attached",
     "darkPactSkeletonLife": "dark_pact_skeleton_life",
+    "deathmarkDeathmarkActive": "deathmark",
+    "feedingFrenzyFeedingFrenzyActive": "feeding_frenzy",
     "heraldOfAgonyVirulenceStack": "herald_of_agony_stacks",
     "iceNovaCastOnFrostbolt": "ice_nova_cast_on_frostbolt",
+    "infusedChannellingInfusion": "infusion",
     "innervateInnervation": "innervate_innervation",
+    "intensifyIntensity": "intensify_stacks",
+    "meatShieldEnemyNearYou": "meat_shield_enemy_nearby",
     "raiseSpectreSpectreLevel": "raise_spectres_spectre_level",
     "raiseSpectreEnableCurses": "raise_spectres_enable_curses",
     "raiseSpectreBladeVortexBladeCount": "raise_spectres_blade_vortex_blade_count",
     "raiseSpectreKaomFireBeamTotemStage": "raise_spectres_kaom_fire_beam_totem_stage",
-    "raiseSpectreEnableSummonedUrsaRallyingCry": "raise_spectres_enable_summoned_ursa_rallying_cry",
+    "raiseSpectreEnableSummonedUrsaRallyingCry":
+        "raise_spectres_enable_summoned_ursa_rallying_cry",
     "raiseSpidersSpiderCount": "raise_spiders_spider_count",
     "siphoningTrapAffectedEnemies": "siphoning_trap_affected_enemies",
+    "bloodSandStance": "stance",
     "summonHolyRelicEnableHolyRelicBoon": "summon_holy_relic_enable_holy_relic_buff",
     "summonLightningGolemEnableWrath": "summon_lightning_golem_enable_wrath",
     "vortexCastOnFrostbolt": "vortex_cast_on_frostbolt",
+    "waveOfConvictionExposureType": "wave_of_conviction_exposure_type",
     "winterOrbStages": "winter_orb_stages",
     "enemyHasPhysicalReduction": "enemy_physical_reduction",
     "enemyIsHexproof": "enemy_hexproof",
@@ -129,6 +212,12 @@ CONFIG_MAP = {
     "useEnduranceCharges": "use_endurance_charges",
     "overrideEnduranceCharges": "max_endurance_charges",
     "useSiphoningCharges": "use_siphoning_charges",
+    "useChallengerCharges": "use_challenger_charges",
+    "overrideChallengerCharges": "max_challenger_charges",
+    "useBlitzCharges": "use_blitz_charges",
+    "overrideBlitzCharges": "max_blitz_charges",
+    "useInspirationCharges": "use_inspiration_charges",
+    "overrideInspirationCharges": "max_inspiration_charges",
     "overrideSiphoningCharges": "max_siphoning_charges",
     "minionsUsePowerCharges": "minions_use_power_charges",
     "minionsUseFrenzyCharges": "minions_use_frenzy_charges",
@@ -140,10 +229,17 @@ CONFIG_MAP = {
     "buffFortify": "fortify",
     "buffTailwind": "tailwind",
     "buffAdrenaline": "adrenaline",
+    "buffDivinity": "divinity",
     "multiplierRage": "rage",
     "conditionLeeching": "leeching",
+    "conditionLeechingLife": "leeching_life",
+    "conditionLeechingEnergyShield": "leeching_energy_shield",
+    "conditionLeechingMana": "leeching_mana",
     "conditionUsingFlask": "using_flask",
     "conditionHaveTotem": "has_totem",
+    "multiplierNearbyAlly": "number_of_nearby_allies",
+    "multiplierNearbyEnemy": "number_of_nearby_enemies",
+    "multiplierNearbyCorpse": "number_of_nearby_corpses",
     "conditionOnConsecratedGround": "on_consecrated_ground",
     "conditionOnBurningGround": "on_burning_ground",
     "conditionOnChilledGround": "on_chilled_ground",
@@ -198,12 +294,14 @@ CONFIG_MAP = {
     "conditionUsedMovementSkillRecently": "used_movement_skill_recently",
     "conditionUsedVaalSkillRecently": "used_vaal_skill_recently",
     "conditionUsedWarcryRecently": "used_warcry_recently",
+    "conditionUsedWarcryInPast8Seconds": "used_warcry_in_past_8_seconds",
     "multiplierMineDetonatedRecently": "number_of_mines_detonated_recently",
     "multiplierTrapTriggeredRecently": "number_of_traps_triggered_recently",
     "conditionConsumedCorpseRecently": "consumed_corpses_recently",
     "multiplierCorpseConsumedRecently": "number_of_corpses_consumed_recently",
     "conditionTauntedEnemyRecently": "taunted_enemy_recently",
-    "conditionBlockedHitFromUniqueEnemyInPast10Sec": "blocked_hit_from_unique_enemy_in_past_ten_seconds",
+    "conditionBlockedHitFromUniqueEnemyInPast10Sec":
+        "blocked_hit_from_unique_enemy_in_past_ten_seconds",
     "critChanceLucky": "lucky_crits",
     "skillChainCount": "number_of_times_skill_has_chained",
     "projectileDistance": "projectile_distance",
@@ -226,6 +324,7 @@ CONFIG_MAP = {
     "conditionEnemyShocked": "enemy_shocked",
     "multiplierFreezeShockIgniteOnEnemy": "enemy_number_of_freeze_shock_ignite",
     "conditionEnemyIntimidated": "enemy_intimidated",
+    "conditionEnemyUnnerved": "enemy_unnerved",
     "conditionEnemyCoveredInAsh": "enemy_covered_in_ash",
     "conditionEnemyRareOrUnique": "enemy_rare_or_unique",
     "enemyIsBoss": "enemy_boss",
@@ -239,8 +338,9 @@ CONFIG_MAP = {
     "enemyConditionHitByLightningDamage": "enemy_hit_by_lightning_damage",
     "EEIgnoreHitDamage": "elemental_equilibrium_ignore_hit_damage",
 }
+# fmt: on
 #: Mapping between PathOfBuilding's export format and this API.
-STATS_MAP = {
+STATS_MAP: Dict[str, str] = {
     "AverageHit": "average_hit",
     "AverageDamage": "average_damage",
     "Speed": "cast_speed",
@@ -327,7 +427,7 @@ STATS_MAP = {
     "ActiveMinionLimit": "active_minion_limit",
 }
 #: Mapping between PathOfBuilding's export format and this API.
-SET_MAP = {
+SET_MAP: Dict[str, str] = {
     "Gloves": "gloves",
     "Weapon 1": "weapon1",
     "Flask 1": "flask1",
